/** This file is part of Domotz Agent.
 * Copyright (C) 2023  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 **/

var DEFAULT_TIMEOUT = 60000;

module.exports.execute = function (resourceLocator, command, timeout, callback, shell) {
    var myConsole = resourceLocator.log.decorateLogs();
    var proc = resourceLocator.utils.proc;

    var stdout = '';
    var stderr = '';

    myConsole.info('executing ' + command);

    var options = {
        timeout: timeout || DEFAULT_TIMEOUT,
        onStdout: function (data) {
            stdout += data;
        },
        onStderr: function (data) {
            stderr += data;
        },
    };
    if (shell) {
        options.shell = true;
    }
    proc.exec(command, options)
        .then(function () {
            callback({ stdout: stdout, stderr: stderr });
        })
        .catch(function (err) {
            callback({ stdout: stdout, stderr: stderr, exit_code: err.code || err.toString() });
        });
};
