/** This file is part of Domotz Agent.
 * Copyright (C) 2023  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 * Measure the baseline latency
 * Created by Stefano Cicero <stefano.cicero@domotz.com> on 25/09/23.
 */

var baselineLatencyMeasurement = function (resourceLocator) {
    var myConsole = resourceLocator.log.decorateLogs();

    const speedTestLatency = resourceLocator.latencyMeasurement;

    if (!speedTestLatency.canPerformLatencyMeasurement()) {
        myConsole.info('Baseline latency measurement disabled');
        return;
    }
    var baselinePingNumber = speedTestLatency.settings.baselinePingNumber;

    speedTestLatency
        .performPingForState('unloaded', baselinePingNumber)
        .then(function () {
            myConsole.info('Baseline latency measurement completed');
        })
        .catch(function () {
            myConsole.error('Unable to complete latency measurement');
        });
};

module.exports.handler = baselineLatencyMeasurement;
