/** This file is part of Domotz Agent.
 * Copyright (C) 2023  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 * Measure the baseline latency
 * Created by Stefano Cicero <stefano.cicero@domotz.com> on 25/09/23.
 */

var counter = 0;

var interfaceListUpdater = function (resourceLocator) {
    var myConsole = resourceLocator.log.decorateLogs();
    const isInterfaceListUpdaterEnabled = resourceLocator.settingsManager.getSetting('configuration.settings.interfaces.update_enabled', true);

    if (!isInterfaceListUpdaterEnabled) {
        return;
    }

    counter++;
    const numberOfFastDiscoveryCycle = resourceLocator.settingsManager.getSetting('configuration.settings.interfaces.update_period', 20);

    if (counter >= numberOfFastDiscoveryCycle) {
        counter = 0;
        myConsole.info('Performing interface list update');
        resourceLocator.discovery.host
            .recoverHostInfoWithRetry()
            .then(function () {
                resourceLocator.settingsManager.loadSettings();
                myConsole.debug('Interface list: ', JSON.stringify(resourceLocator.discovery.host.getInterfacesList()));
            })
            .catch(function () {
                myConsole.error('Unable to retrieve host info');
            });
    } else {
        myConsole.info('Called Interface List Updater task: ' + counter + ' times, starting at ' + numberOfFastDiscoveryCycle);
    }
};

module.exports.handler = interfaceListUpdater;
