/** This file is part of Domotz Agent.
 * Copyright (C) 2021  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Module entry point
 */

function factory(resourceLocator) {
    var platform = resourceLocator.utils.platform;
    var q = resourceLocator.q;
    var childProcess = resourceLocator.utils.childProcess;

    function _parseWinrmIdOutput(data) {
        const regex = new RegExp('Stack:\\s*([\\d.]+)');
        const match = data.match(regex);
        return match ? match[1].trim() : null;
    }

    function getVersion() {
        if (!platform.isWindows()) {
            return q.reject();
        }
        var deferred = q.defer();
        childProcess.exec('powershell.exe "winrm id"', function (err, stdout, stderr) {
            if (err || stderr) {
                deferred.reject();
            } else {
                deferred.resolve(_parseWinrmIdOutput(stdout));
            }
        });
        return deferred.promise;
    }

    return {
        createRemote: function (host, port, username, password) {
            return require('./remote').Remote(host, port, username, password, resourceLocator);
        },
        getVersion: getVersion,
    };
}

module.exports.factory = factory;
